#!/usr/bin/python
# -*- coding: iso-8859-1 -*-
"""
(c) Copyright 2015 Hexoskin
Permission to use, copy, modify, and distribute this software for any purpose with or without fee is hereby granted,
provided that the above copyright notice and this permission notice appear in all copies. The software is provided
"as is" and the author disclaims all warranties with regard to this software including all implied warranties of
merchantability and fitness. In no event shall the author be liable for any special, direct, indirect, or
consequential damages or any damages whatsoever resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortious action, arising out of or in connection with the use or performance of this
software.
"""

import Tkinter as tk
import ttk
import os
from tkFileDialog import askdirectory
from tkMessageBox import askokcancel
from tkFont import Font
from convert_raw_data import convert_directory, DEFAULT_SETTINGS, VERSION
from api_request import AuthorisationError, get_url, download_data
import json
from StringIO import StringIO
import zipfile
from datetime import timedelta
from os.path import expanduser
import datetime
from dateutil.tz import tzlocal
from dateutil.parser import parse
import time
import pytz
from dateutil.relativedelta import relativedelta

count_range_record = 10
listbox_height = 10
delta = relativedelta(years=1)
start_2010 = 323149824000

def get_script_dir(follow_symlinks=True):
    path1 = expanduser("~")
    if os.path.exists(os.path.join(path1, 'Downloads')):
        path1 = os.path.join(path1, 'Downloads')
    return path1


def date_to_hxts(dt):
    dt = dt.replace(tzinfo=tzlocal())
    return int((time.mktime(dt.utctimetuple()) + dt.microsecond/1000000.) * 256)



class simpleapp_tk(tk.Tk):
    def __init__(self, parent):
        tk.Tk.__init__(self, parent)
        self.parent = parent
        self.font_large= Font(weight="bold")
        self.settings = DEFAULT_SETTINGS
        self.initialize()
        self.login = None
        self.user = None # selected user simon.dubeau@carretechnologies.com


    def update_start_button(self, extra=None, state=tk.NORMAL):
        if self.notebook.index(self.notebook.select()) == 0:
            #self.button_download_convert.configure(text='Download')
            #self.button_download_convert.configure(state=tk.NORMAL if len(self.api_listbox.curselection()) and state == tk.NORMAL else tk.DISABLED)
            self.button_download.configure(state=tk.NORMAL if len(self.api_listbox.curselection()) and state == tk.NORMAL else tk.DISABLED)
            self.button_convert.configure(state=tk.NORMAL if len(self.api_listbox.curselection()) and state == tk.NORMAL else tk.DISABLED)

        else:
            #self.button_download_convert.configure(text='Convert')
            self.button_download.configure(state=tk.NORMAL if len(self.paths_listbox.curselection()) and state == tk.NORMAL else tk.DISABLED)
            self.button_convert.configure(state=tk.NORMAL if len(self.paths_listbox.curselection()) and state == tk.NORMAL else tk.DISABLED)


    def initialize(self):
        self.notebook = ttk.Notebook(self, name='notebook')
        f_download = tk.Frame(self.notebook)
        self.notebook.add(f_download, text='Download', state='normal')
        f_convert = tk.Frame(self.notebook)
        self.f_convert = f_convert
        self.notebook.add(f_convert, text='Convert', state='normal')
        self.notebook.enable_traversal()
        self.notebook.bind('<<NotebookTabChanged>>', self.set_path_listbox)
        self.notebook.pack(fill=tk.BOTH, expand=tk.Y, padx=2, pady=3)
        self.notebook = self.notebook


        ## frame api
        tk_frame_api = tk.Frame(f_download)
        self.button_user = tk.Button(tk_frame_api, text='Change user',  command=self.get_server_user)
        self.button_user.pack(fill=tk.BOTH, side=tk.LEFT)
        self.button_range = tk.Button(tk_frame_api, text='Fetch Range/activitytype',  command=self.get_server_ranges)
        self.button_range.pack(fill=tk.BOTH, side=tk.LEFT)

        self.button_record = tk.Button(tk_frame_api, text='Fetch records',  command=self.get_server_records)
        self.button_record.pack(fill=tk.BOTH, side=tk.LEFT)
        tk_frame_api.pack(side=tk.TOP, fill=tk.BOTH)

        tk_frame_api2 = tk.Frame(f_download)
        self.user_label = tk.Label(tk_frame_api2, text='user: ')
        self.user_label.pack(fill=tk.BOTH, side=tk.LEFT)
        tk.Label(tk_frame_api2, text='start: ').pack(fill=tk.BOTH, side=tk.LEFT)
        var = tk.StringVar()
        self.text_start = DateEntry(tk_frame_api2, bd=1, width=30, textvariable=var, command=self.date_changed)
        self.text_start.pack(fill=tk.BOTH, side=tk.LEFT)
        self.text_start.insert(tk.END, str(datetime.datetime.now().replace(microsecond=0) - delta))
        tk.Label(tk_frame_api2, text='end: ').pack(fill=tk.BOTH, side=tk.LEFT)
        var = tk.StringVar()
        self.text_end = DateEntry(tk_frame_api2, bd=1, width=30, textvariable=var, command=self.date_changed)
        self.text_end.pack(fill=tk.BOTH, side=tk.LEFT)
        self.text_end.insert(tk.END, str(datetime.datetime.now().replace(microsecond=0)))
        tk_frame_api2.pack(side=tk.TOP, fill=tk.BOTH)

        tk_frame_listbox = tk.Frame(f_download)
        tk_frame_listbox_l = tk.Frame(tk_frame_listbox)
        tk_frame_listbox_r = tk.Frame(tk_frame_listbox, width=10)

        self.api_listbox = tk.Listbox(tk_frame_listbox_r, exportselection=False, selectmode=tk.MULTIPLE, width=100, height=listbox_height)
        self.api_listbox.bind("<<ListboxSelect>>", self.update_start_button)
        self.api_listbox.pack(side=tk.TOP, fill=tk.BOTH)
        tk_frame_api2 = tk.Frame(f_download)
        self.button_previous = tk.Button(tk_frame_listbox_l, text=u"\u25B3  \u25B3 ", wraplength=2, width=2, command=self.previous, state=tk.DISABLED)
        self.button_previous.pack(fill=tk.BOTH, side=tk.TOP)

        self.button_next = tk.Button(tk_frame_listbox_l, text=u"\u25BD \u25BD ", wraplength=2, width=2, command=self.next, state=tk.DISABLED)
        self.button_next.pack(fill=tk.BOTH, side=tk.TOP)
        tk_frame_api2.pack(side=tk.TOP, fill=tk.BOTH)
        tk_frame_listbox_l.pack(side=tk.LEFT)
        tk_frame_listbox_r.pack(side=tk.LEFT,  fill=tk.BOTH)
        tk_frame_listbox.pack(side=tk.TOP, fill=tk.BOTH)

        button_var = tk.IntVar()
        button_var.set(1)
        self.cb_convert_after = tk.Checkbutton(f_download, variable=button_var,
                                       text='Convert after downloading raw data')
        self.cb_convert_after.var = button_var
        self.cb_convert_after.pack(side=tk.TOP)
        self.button_download = tk.Button(f_download, text='Download',
                                      command=self.download_data, state=tk.DISABLED)
        self.button_download.pack(fill=tk.BOTH, side=tk.LEFT)

        ## frame convert
        tk_frame_button = tk.LabelFrame(f_convert, text='Quality and Group options', font=self.font_large)
        button_var = tk.IntVar()
        button_var.set(0)
        self.cb_group = tk.Checkbutton(f_convert, variable=button_var, command=self.changeGroup,
                                       text='Group 1 Hz data in outputData.csv instead of outputting them in different files')
        self.cb_group.var = button_var
        self.cb_group.pack(fill=tk.BOTH, side=tk.TOP)

        button_var = tk.IntVar()
        button_var.set(0)

        button_var = tk.IntVar()
        button_var.set(0)
        self.cb_quality = tk.Checkbutton(f_convert, variable=button_var, command=self.changeQuality,
                                         text='Add quality status to heartrate, breathingrate and rr_interval instead of putting them in separate files ')
        self.cb_quality.var = button_var
        self.cb_quality.pack(fill=tk.BOTH, side=tk.TOP)

        button_var = tk.IntVar()
        button_var.set(0)
        self.cb_epoch = tk.Checkbutton(f_convert, variable=button_var, text='Save in Posix epoch time. (seconds since 1 January 1970)')
        self.cb_epoch.var = button_var
        self.cb_epoch.pack(fill=tk.BOTH, side=tk.TOP)

        button_var = tk.IntVar()
        button_var.set(1)
        self.cb_skip_raw = tk.Checkbutton(f_convert, variable=button_var, command=self.changeSkip, text='Skip raw data (with frequency > 1Hz).')
        self.cb_skip_raw.var = button_var
        self.cb_skip_raw.pack(fill=tk.BOTH, side=tk.TOP)

        button_var = tk.IntVar()
        button_var.set(1)
        self.cb_header = tk.Checkbutton(f_convert, variable=button_var,  text="Save header")
        self.cb_header.var = button_var
        self.cb_header.pack(fill=tk.BOTH, side=tk.TOP)

        button_var = tk.IntVar()
        button_var.set(0)
        self.cb_as_nan = tk.Checkbutton(f_convert, variable=button_var, state=tk.DISABLED, text="Save empty values as 'nan'")
        self.cb_as_nan.var = button_var
        self.cb_as_nan.pack(fill=tk.BOTH, side=tk.TOP)
        tk_frame_button.pack(side=tk.TOP, fill=tk.BOTH)
        tk_frame_directories = tk.LabelFrame(f_convert, text='Directories', font=self.font_large)
        self.paths_listbox = tk.Listbox(f_convert, exportselection=False, selectmode=tk.MULTIPLE, height=listbox_height)
        self.paths_listbox.bind("<<ListboxSelect>>", self.update_start_button, '+')
        self.paths_listbox.pack(side=tk.TOP, fill=tk.BOTH)
        tk_frame_directories.pack(side=tk.TOP, fill=tk.BOTH)
        self.button_convert = tk.Button(f_convert, text='Convert',
                                      command=self.convert_data, state=tk.DISABLED)
        self.button_convert.pack(fill=tk.BOTH, side=tk.LEFT)

        ## frame path
        tk_frame_path = tk.LabelFrame(self, text='Data directory', bd=2)
        self.button_browse = tk.Button(tk_frame_path, text='Browse', command=self.load_settings, width=10)
        self.button_browse.pack(side=tk.LEFT)
        tk_frame_path2 = tk.Frame(tk_frame_path,  bd=2,  relief=tk.SUNKEN)
        self.text_path = tk.Text(tk_frame_path2, height=1, width=90)
        self.text_path.insert(tk.END, get_script_dir())
        self.text_path.pack(side=tk.LEFT, fill=tk.BOTH)
        tk_frame_path2.pack(side=tk.LEFT, fill=tk.BOTH)
        tk_frame_path.pack(side=tk.TOP, fill=tk.BOTH)


        ## frame  and log
        self.frame_log = tk.LabelFrame(self, text='Execution and logging', bd=2)
        self.frame_start_button = tk.Frame(self.frame_log)

        self.button_help = tk.Button(self.frame_start_button,  text="Help",  command=self.help)
        self.button_help.pack(side=tk.LEFT)

        self.button_end = tk.Button(self.frame_start_button,  text="QUIT", command=self.quit)
        self.button_end.pack(side=tk.LEFT, fill=tk.BOTH)

        self.progressbar = ttk.Progressbar(self, orient='horizontal', mode='determinate')
        self.progressbar.pack(in_=self.frame_start_button, side=tk.LEFT)
        self.progressbar.lower(self.frame_start_button)
        self.frame_start_button.pack(fill=tk.BOTH, side=tk.TOP)
        self.text_log = tk.Text(self.frame_log, height=15)
        self.text_log.tag_config("warn", foreground="orange")
        self.text_log.tag_config("error", foreground="red")
        self.text_log.pack(side=tk.LEFT, fill=tk.BOTH)
        self.frame_log.pack(fill=tk.BOTH, side=tk.TOP)

    def get_duration_str(self, mode='range'):
        start = self.text_start.get()
        end = self.text_end.get()
        try:
            start = date_to_hxts(parse(start))
        except:
            self.log('invalid start.  No time limit is applied')
            return ''
        try:
            end = date_to_hxts(parse(end))
        except:
            self.log('invalid end. No time limit is applied')
            return ''
        if mode == 'range':
            return '&start__gte={}&end__lte={}'.format(start, end)
        else:
            return '&start={}&end={}'.format(start, end)

    def date_changed(self, event=None):
        if hasattr(self, 'last_fetch'):
            self.last_fetch()

    def get_server_ranges(self):
        self.last_fetch = self.get_server_ranges
        self.check_server_user()
        self.get_server_data('/api/range/?limit={}&offset={}&user={}&order_by=start{}'.format(count_range_record, 0,
                                                                                              self.user[1],
                                                                                              self.get_duration_str()))


    def get_server_records(self):
        self.last_fetch = self.get_server_records
        self.check_server_user()
        self.get_server_data('/api/record/?limit={}&offset={}&user={}&order_by=start{}'.format(count_range_record, 0,
                                                                                               self.user[1],
                                                                                               self.get_duration_str()))

    def get_server_user(self):
        if self.check_login():
            self.set_enable(tk.DISABLED)
            self.update()
            output = get_url(user=self.login[0], password=self.login[1], apiurl='/api/user/?limit=0')
            output = json.loads(output)['objects']
            self.set_enable(tk.NORMAL)
            user_window = SelectUser(self, output)
            if user_window.winfo_exists():
                self.wait_window(user_window)
            return True

    def check_server_user(self):
        if not self.user:
            self.get_server_user()


    def get_server_data(self, apiurl):
        if self.check_login():
            self.set_enable(tk.DISABLED)
            self.update()
            output = get_url(user=self.login[0], password=self.login[1], apiurl=apiurl)
            self.set_download_listbox(output)
            self.set_next_previous(output)
            self.set_enable(tk.NORMAL)


    def next(self):
        self.get_server_data(self.button_next.apiurl)


    def previous(self):
        self.get_server_data(self.button_previous.apiurl)


    def set_next_previous(self, output):
        output = json.loads(output)['meta']
        self.button_next.configure(state=tk.NORMAL if output['next'] else tk.DISABLED)
        self.button_previous.configure(state=tk.NORMAL if output['previous'] else tk.DISABLED)
        self.button_next.apiurl = output['next']
        self.button_previous.apiurl = output['previous']


    def get_duration(self, a):
        return str(timedelta(seconds=(a['end']-a['start'])/256))

    def _get_user_id(self, a):
        if isinstance(a['user'], dict):
            return a['user']['id']
        else:
            return int(a['user'].split('/')[-2])

    def get_user(self, a):
        return 'user={}'.format(self._get_user_id(a))


    def get_name(self, a):
        return a.get('name', '')[:32]

    def get_start(self, a):
        return str(parse(a['start_date'][:-3]).astimezone(tzlocal()).replace(tzinfo=None))

    def set_download_listbox(self, output):
        self.api_listbox.objects = json.loads(output)['objects']

        rngs = [(a['resource_uri'].split('/')[2], a['resource_uri'].split('/')[3], self.get_start(a), self.get_duration(a), a['status'], self.get_name(a)) for a in self.api_listbox.objects]
        self.api_listbox.delete(0, tk.END)
        for rng in rngs:
            self.api_listbox.insert(tk.END, '    '.join(rng))
        if len(rngs) == 0:
            self.log('No data found')


    def set_path_listbox(self, event=None):
        path1 = self.text_path.get("1.0", 'end-1c')
        self.paths_listbox.delete(0, tk.END)
        for subpath in os.listdir(path1):
            if (subpath.startswith('range') or subpath.startswith('record') or subpath.startswith('datatype')
                or subpath.startswith('user') or
                    os.path.exists(os.path.join(subpath, 'info.json'))) and os.path.isdir(os.path.join(path1, subpath)):
                self.paths_listbox.insert(tk.END, subpath)
        self.update_start_button()
        self.update()


    def unzip(self, path1, zip_content):
        stringio = StringIO(zip_content)
        zz = zipfile.ZipFile(stringio)
        zz.extractall(path1)


    def download_data(self):
        if self.check_login():
            self.set_enable(tk.DISABLED)
            selects = [int(a) for a in self.api_listbox.curselection()]
            self.progressbar.configure(maximum=len(selects))
            self.progressbar.lift(self.frame_start_button)
            self.update()
            for select_id in selects:
                type_ = self.api_listbox.objects[select_id]['resource_uri'].split('/')[2]
                id_ = self.api_listbox.objects[select_id]['resource_uri'].split('/')[3]
                user = self._get_user_id(self.api_listbox.objects[select_id])
                try:
                    str_ = '{}_{}'.format(type_, id_)
                    path1 = os.path.join(self.text_path.get("1.0", 'end-1c') , 'user_{}'.format(user))
                    if os.path.exists(path1) and  [a for a in os.listdir(path1) if a.startswith(str_)]:
                        dir = [a for a in os.listdir(path1) if a.startswith(str_)][0]
                        self.log('Directory already present not redownloaded {}'.format(dir))
                    else:
                        self.log('downloading {} {}'.format(type_, id_))
                        resp = download_data(user=self.login[0], password=self.login[1], filters='{}={}'.format(type_, id_))
                        self.unzip(path1, resp)
                        dirs = [a for a in os.listdir(path1) if a.startswith(str_)]
                        if dirs:
                            if not os.path.exists(os.path.join(path1, 'info.json')):
                                with open(os.path.join(path1, 'info.json'), 'w') as f:
                                    json.dump(dict(decoder_version='0.0.0', timestamp=start_2010/256, start=start_2010,
                                                   start_date= "2010-01-01T00:00:00.000000",
                                                   note="Automatically generated by HxConvertSourceFile to be able to convert a complete user directory"),f, indent=4)
                            if self.cb_convert_after.var.get() == 1:
                                self.convert_path_raw(os.path.join(path1, dirs[0]))
                except Exception as e:
                    self.log_error(str(e))
                self.progressbar.step()
                self.progressbar.update()
                self.update()
            self.progressbar.configure(maximum=len(selects))
            self.progressbar.lower(self.frame_start_button)
            self.set_enable(tk.NORMAL)


    def convert_data(self):
        self.set_enable(tk.DISABLED)
        selects = [self.paths_listbox.get(int(a)) for a in self.paths_listbox.curselection()]
        path1 = self.text_path.get("1.0", 'end-1c')
        self.progressbar.configure(maximum=len(selects))
        self.progressbar.lift(self.frame_start_button)
        self.update()
        for subdir in selects:
            try:
                self.convert_path_raw(os.path.join(path1, subdir))

            except Exception as e:
                self.log_error(str(e))
            self.progressbar.step()
            self.update()
            self.log('----')
        self.progressbar.lower(self.frame_start_button)
        self.set_enable(tk.NORMAL)


    def set_enable(self, state=None):
        self.button_convert.configure(state=state)
        self.button_end.configure(state=state)
        self.button_download.configure(state=state)
        self.button_range.configure(state=state)
        self.button_user.configure(state=state)
        self.button_browse.configure(state=state)
        self.update_start_button(state=state)


    def changeGroup(self, state=None):
        self.cb_quality.var.set(0)
        self.changeSkip()

    def changeSkip(self, state=None):
        if self.cb_group.var.get() == 0 or self.cb_skip_raw.var.get() == 0:
            self.cb_as_nan.var.set(0)
            self.cb_as_nan.configure(state=tk.DISABLED)
        else:
            self.cb_as_nan.configure(state=tk.NORMAL)


    def changeQuality(self, state=None):
        self.cb_group.var.set(0)


    def get_settings(self, state=None):
        self.settings['add_quality'] = self.cb_quality.var.get()
        self.settings['group_raw'] = self.cb_group.var.get() and not self.cb_skip_raw.var.get()
        self.settings['group'] = self.cb_group.var.get()
        self.settings['epoch'] = self.cb_epoch.var.get()
        self.settings['header'] = self.cb_header.var.get()
        self.settings['skip_raw'] = self.cb_skip_raw.var.get()
        self.settings['as_nan'] = self.cb_as_nan.var.get()
        return self.settings






    def help(self, state=None):

        help_text = [('This program convert the Hexoskin raw binary data to .csv file.\n' , 'normal'),
                     ('version : {}'.format('.'.join([str(i) for i in VERSION])), 'bold'),
                     ('\n\nFor more info about the datatype, see: ', 'normal'),
                     ("https://api.hexoskin.com/docs/resource/datatype/", "italic"),
                     ('.\n\nIf the data are non-contiguous, the data will be downloaded in separated directories '
                      'but will be regrouped in the parent directory during conversion.', 'normal'),
                     ('\n\nStart and end are defined using the local timezone.', 'normal'),
                     ('\n\nProgram downloaded from: ', 'normal'),
                     ('http://support.hexoskin.com/customer/portal/articles/1758581-how-do-i-convert-binary-data-downloaded-from-the-dashboard-', 'italic'),
                     ('.', 'normal'),
                     ]

        f = Font(family="times",  slant="italic", underline=1)
        f2 = Font(family="times", slant='roman', underline=0)
        f3 = Font(family="times", weight='bold', )
        toplevel = tk.Toplevel()
        label1 = tk.Text(toplevel, height=15, width=60, borderwidth=3)
        label1.tag_config("italic", font=f)
        label1.tag_config("normal", font=f2)
        label1.tag_config("bold", font=f3)
        for str_, font in help_text:
            label1.insert(tk.END, str_, font)
        label1.pack()


    def check_login(self):
        if not self.login:
            pwd_window = Password(self)
            self.wait_window(pwd_window)
        return self.login


    def load_settings(self, state=None):
        inputDir = self.text_path.get("1.0", 'end-1c')
        inputDir = inputDir if os.path.exists(inputDir) else ''
        filename = askdirectory(initialdir=inputDir)
        self.text_path.delete(1.0, tk.END)
        self.text_path.insert(tk.END, filename)
        self.set_path_listbox()


    def check_user_dir(self, inputDir):
        """Ask to confirm if we do processing on a user directory containing subdirs"""
        #if os.path.split(inputDir)[-1].startswith('user'):
        path1 = os.path.join(inputDir, 'info.json')
        if os.path.exists(path1):
            with open(path1, 'r') as f:
                data = json.load(f)
            if data['start'] == start_2010:
                l = len([a for a in os.listdir(inputDir) if os.path.isdir(os.path.join(inputDir, a))])
                if l:
                    return askokcancel("Exit", "Do you want to convert the directory {} containing {} subdirectories".format(inputDir, l))
        return True


    def convert_path_raw(self, inputDir):
        inputDir = inputDir or self.text_path.get("1.0", 'end-1c')
        settings = self.get_settings()
        if not os.path.exists(inputDir):
            self.logWarning('Input directory does not exist!')
            return
        self.log('Converting directory {}'.format(inputDir))
        if not settings['skip_raw']:
            self.log('Can take up to 1 minutes per hour of data.')
        if self.check_user_dir(inputDir):
            try:
                convert_directory(inputDir, settings)
            except Exception as e:
                self.log_error('An error occured: %s' % (e))


    def logWarning(self, msg):
        self.log(msg)
        self.text_log.tag_add("warn", "end-{}c".format(len(msg)+2), "end-1c")


    def log_error(self, msg):
        self.log(msg)
        self.text_log.tag_add("error", "end-{}c".format(len(msg)+2), "end-1c")


    def log(self, msg):
        #to_del = int(self.text_log.index('end').split('.')[0]) -15
        #if to_del > 1:
        #    self.text_log.delete('1.0', '{}.0'.format(to_del))
        print msg
        self.text_log.insert(tk.END, msg+ '\n')
        self.text_log.see(tk.END)
        self.update()



class SelectUser(tk.Toplevel):
    def __init__(self, parent, output=None):
        tk.Toplevel.__init__(self, parent)
        self.title('Select user')
        self.parent = parent
        self.api_listbox = tk.Listbox(self, exportselection=False, width=100, height=listbox_height)
        self.api_listbox.objects = sorted(output, key=lambda x: x['username'])
        self.api_listbox.pack(side=tk.TOP)
        self.set_download_user_listbox()
        self.api_listbox.selection_set(0)
        b = tk.Button(self, borderwidth=4, text="Accept", width=10, pady=8, command=self.accept)
        b.pack(side=tk.TOP)
        if len(output) == 1:
            self.parent.log('Only one user, it is selected')
            self.accept()

    def accept(self, event=None):
        user = self.api_listbox.objects[int(self.api_listbox.curselection()[0])]
        self.parent.user = user['username'], user['id']
        self.parent.user_label.config(text='user: {0[0]} {0[1]}'.format(self.parent.user))
        self.destroy()

    def set_download_user_listbox(self):
        usrs = [(str(a['id']), str(a['username'])) for a in self.api_listbox.objects]
        self.api_listbox.delete(0, tk.END)
        for user in usrs:
            self.api_listbox.insert(tk.END, '    '.join(user))

class DateEntry(tk.Entry):
    # base class for validating entry widgets
    maxlength = 19
    def __init__(self, master, value="", command=None, **kw):
        apply(tk.Entry.__init__, (self, master), kw)
        self.master = master
        self.__value = value
        self.__variable = tk.StringVar()
        self.__variable.set(value)
        self.__variable.trace("w", self.__callback)
        self.config(textvariable=self.__variable)
        self.command = command

    def __callback(self, *dummy):
        value = self.__variable.get()
        newvalue = self.validate(value)
        if newvalue is None:
            self.__variable.set(self.__value)
        elif newvalue != value:
            self.__value = newvalue
            self.__variable.set(self.newvalue)
            self.on_valid()
        else:
            self.__value = value
            self.on_valid()

    def on_valid(self):
        try:
            if 2010 < parse(self.__value).year< 2100:
                self.command()
        except ValueError:
            pass

    def validate(self, value):
        # override: return value, new value, or None if invalid
        if (self.maxlength is None or len(value) <= self.maxlength) and \
                not any(c.isalpha() for c in value):
            return value

        return None # new value too long


class Password(tk.Toplevel):
    failure_max = 3

    def __init__(self, parent):
        tk.Toplevel.__init__(self, parent)
        #self.geometry('300x160')
        self.minsize(width=300, height=160)
        self.maxsize(width=300, height=160)
        self.parent = parent
        self.title('Enter your information')

        #entrys with not shown text
        self.user = self.make_entry("User name:", 16)
        self.password = self.make_entry("Password:", 16, show="*")
        #button to attempt to login
        b = tk.Button(self, borderwidth=4, text="Login", width=10, pady=8, command=self.check_password)
        b.pack(side=tk.BOTTOM)
        self.password.bind('<Return>', self.enter)
        self.user.focus_set()
        self.failures = 0


    def make_entry(self, caption, width=None, **options):
        tk.Label(self, text=caption).pack(side=tk.TOP)
        entry = tk.Entry(self, **options)
        if width:
            entry.config(width=width)
        entry.pack(side=tk.TOP, padx=10, fill=tk.BOTH)
        return entry

    def enter(self, event):
        self.check_password()

    def check_password(self):
        """ Collect 1's for every failure and quit program in case of failure_max failures """
        try:
            resp = get_url(user=self.user.get(), password=self.password.get(), apiurl='/api/?limit=1')
            self.parent.log('Logged in')
            self.parent.login = self.user.get(), self.password.get()
            self.destroy()
            return
        except AuthorisationError as e:
            self.parent.log(str(e))
            self.destroy()
            return
        except AssertionError:
            self.failures += 1
        if self.failures == self.failure_max:
            self.parent.log('Unauthorized login attempt')
            self.destroy()
            #raise SystemExit('Unauthorized login attempt')
        else:
            self.title('Try again. Attempt %i/%i' % (self.failures + 1, self.failure_max))


def run_GUI():
    app = simpleapp_tk(None)
    app.title('Hexoskin Convertion from RAW to .csv files')
    app.mainloop()

if __name__ == "__main__":
    run_GUI()


